import os
import sys
from argparse import ArgumentParser

from ximilar.client import RecognitionClient
from ximilar.client.constants import FILE, DEFAULT_WORKSPACE, NORESIZE
from ximilar.client.recognition import Image

if __name__ == "__main__":
    parser = ArgumentParser(description="Delete all images from a workspace")
    parser.add_argument("--api_prefix", type=str, help="API prefix", default="https://api.ximilar.com/")
    parser.add_argument("--auth_token", help="user authorization token to be used for API authentication")
    parser.add_argument("--workspace_id", help="ID of workspace to upload the images into", default=DEFAULT_WORKSPACE)
    args = parser.parse_args()

    client = RecognitionClient(token=args.auth_token, endpoint=args.api_prefix, workspace=args.workspace_id)

    images, next_page, status = client.get_training_images()
    while images:
        image: Image
        for image in images:
            print("removing image: " + str(image.id))
            image.remove()

        if not next_page:
            break
        images, next_page, status = client.get_training_images(next_page)



import requests
import json
import base64
import cv2
import os
import numpy as np
import concurrent.futures

from tqdm import tqdm

from ximilar.client.constants import *

CONFIG_ENDPOINT = "account/v2/config/"


class RestClient(object):
    """
    Parent class that implements HTTP GET, POST, DELETE methods with requests lib and loading images to base64.

    All objects contains TOKEN and ENDPOINT information.
    """

    def __init__(self, token, endpoint=ENDPOINT, max_image_size=600, resource_name=""):
        self.token = token
        self.cache = {}
        self.endpoint = endpoint
        self.max_image_size = max_image_size
        self.headers = {"Content-Type": "application/json", "Authorization": "Token " + self.token}
        self.check_resource(resource_name)

    def invalidate(self):
        self.cache = {}

    def _type(self):
        """
        Returns the Class name of the object.
        For example RecogntionClient._type() => "RecognitionClient"
                    Image._type() => "Image"
        """
        return self.__class__.__name__

    def get(self, api_endpoint, data=None, params=None):
        """
        Call the http GET request with data.
        :param api_endpoint: endpoint path
        :param data: optional data
        :param params: optional dictionary of URL params
        :return: json response
        """
        result = requests.get(self.endpoint + api_endpoint, params=params, headers=self.headers, data=data, timeout=30)
        return result.json()

    def post(self, api_endpoint, data=None, files=None, params=None, method=requests.post):
        """
        Call the http POST request with data.

        :param api_endpoint: endpoint path
        :param data: optional data
        :param files: optional files to upload
        :param params: optional dictionary of URL params
        :param method: POST or PUT method of request
        :return: json response
        """
        self.invalidate()

        if data is not None and files is not None:
            raise Exception("Unable to send data along with files with python requests library!")

        if data is not None:
            data = json.dumps(data)

        result = method(
            self.endpoint + api_endpoint, params=params, headers=self.headers, data=data, files=files, timeout=30
        )

        try:
            json_result = result.json()
            return json_result
        except ValueError as e:
            return None

    def put(self, api_endpoint, data=None, files=None, params=None):
        """
        Call the http PUT request with data
        """
        return self.post(api_endpoint, data=data, files=files, params=params, method=requests.put)

    def delete(self, api_endpoint, data=None, params=None):
        """
        Call the http DELETE request with data.
        :param api_endpoint: endpoint path
        :param data: optional data
        :param params: optional dictionary of URL params
        :return: response
        """
        self.invalidate()

        result = requests.delete(
            self.endpoint + api_endpoint, params=params, headers=self.headers, data=data, timeout=30
        )

        if result.status_code == HTTP_NO_COTENT_204:
            return result

        return result.json()

    def check_resource(self, resource_name):
        """
        Checks if the user has access (resource) to the service.
        :param resource_name: name of the service
        :return: True if user has access otherwise False
        """
        if "localhost" in self.endpoint:
            return True

        # we don't want to check resource for entities like Image, Task, Object, DetectionLabel, ...
        if "Client" not in self._type():
            return True

        # we need to authorize it with FIXED Endpoint https://api.ximilar.com/authorization/v2/authorize
        # as the self.endpoint can be different size
        result = requests.post(
            ENDPOINT + "authorization/v2/authorize",
            data=json.dumps({"service_name": resource_name}),
            headers=self.headers,
            timeout=10,
        )
        try:
            result = result.json()
        except:
            result = None

        if result and USER_ID in result:
            return True

        raise Exception("User has no access for service: " + resource_name + ". Please contact tech@ximilar.com!")

    def get_all_paginated_items(self, url):
        """
        Getting all paginated items from specific endpoint url
        :param url: url path which will be queried
        :return: items
        """
        items = []

        while True:
            result = self.get(url)
            if RESULTS in result:
                for item in result[RESULTS]:
                    items.append(item)

                if result[NEXT] is None:
                    break
            else:
                if DETAIL in result:
                    return None, {DETAIL: result[DETAIL], STATUS: STATUS_ERROR}
                else:
                    return None, RESULT_ERROR

            # we need to replace the full url just with the end, because endpoint
            # is automatically adde during the request
            url = (
                result[NEXT]
                .replace(self.endpoint, "")
                .replace(self.endpoint.replace("https", "http"), "")
                .replace("http://localhost/api/", "")
            )

        return items, RESULT_OK

    def resize_image_data(self, image_data, aspect_ratio=False, resize=True):
        """
        Resize image data that are no bigger than max_size.
        :param image_data: cv2/np ndarray
        :return: cv2/np ndarray
        """
        # do not resize image if set to 0
        if not resize or self.max_image_size == 0:
            return image_data

        height, width, _ = image_data.shape
        if height > self.max_image_size and width > self.max_image_size and not aspect_ratio:
            image_data = cv2.resize(image_data, (self.max_image_size, self.max_image_size))
        if height > self.max_image_size and width > self.max_image_size and aspect_ratio:
            image_data = cv2.resize(image_data, self.get_aspect_ratio_dim(image_data, self.max_image_size))
        return image_data

    def get_aspect_ratio_dim(self, image, img_size):
        if image.shape[0] > image.shape[1]:
            r = float(img_size) / image.shape[1]
            dim = (img_size, int(image.shape[0] * r))
        else:
            r = float(img_size) / image.shape[0]
            dim = (int(image.shape[1] * r), img_size)
        return dim

    def cv2_imread(self, path):
        image = cv2.imread(str(path))
        return image

    def cv2_imwrite(self, image_record, path):
        """
        Writes image data into given file
        :param image_record: dictionary with image data and color space
        :param path: file to store the image to
        :return: true if everything was fine
        """
        image = image_record[IMG_DATA]
        image_space = image_record[COLOR_SPACE] if COLOR_SPACE in image_record else "RGB"
        image = self._convert_image_to_bgr(image, image_space)

        cv2.imwrite(path, image)
        return True

    def load_base64_file(self, path, resize=True):
        """
        Load file from disk to base64.
        :param path: local path to the image
        :return: base64 encoded string
        """
        image = self.cv2_imread(path)
        image = self.resize_image_data(image, resize=resize)
        image = self.cv2img_to_base64(image, image_space="BGR", resize=resize)
        return image

    def cv2img_to_base64(self, image, image_space="RGB", resize=True):
        """
        Load raw numpy/cv2 data of image to base64. The input image to this method should have RGB order.
        The ximilar accepts base64 data to have BGR order that is why we convert it here.
        The image_data was loaded in similar way:
            image = cv2.imread(str(path))
            image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

        :param image: numpy/cv2 data with RGB order
        :param resize: if we want to resize image
        :return: base64 encoded string
        """
        image = self._convert_image_to_bgr(image, image_space)
        image = self.resize_image_data(image, resize=resize)
        retval, buffer = cv2.imencode(".jpg", image)
        jpg_as_text = base64.b64encode(buffer).decode("utf-8")
        return jpg_as_text

    def load_url_image(self, path, resize=True):
        """
        Load url file to base64 WITHOUT resizing (it is used in upload image for recognition).
        :param path: url path
        :param resize: if we want to resize image
        :return: base64 encoded string
        """
        r = requests.get(str(path), headers={"Accept": "*/*", "User-Agent": "request"})
        img_bin = r.content
        image = np.asarray(bytearray(img_bin), dtype="uint8")
        image = cv2.imdecode(image, cv2.IMREAD_COLOR)
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
        image = self.cv2img_to_base64(image, resize=resize)
        return image

    def preprocess_records(self, records):
        """
        Preprocess all records (list of dictionaries with possible '_base64'|'_file'|'_url' fields
        before processing/upload to Ximilar Application.
        :param records: list of dictionaries
        :return: modified list of dictionaries
        """
        for i in range(len(records)):
            if FILE in records[i] and BASE64 not in records[i] and IMG_DATA not in records[i]:
                records[i][BASE64] = self.load_base64_file(records[i][FILE])
            elif IMG_DATA in records[i]:
                records[i][BASE64] = self.cv2img_to_base64(
                    records[i][IMG_DATA], image_space=records[i][COLOR_SPACE] if COLOR_SPACE in records[i] else "RGB"
                )

            # finally we need to delete the image data and just send url or base64
            if IMG_DATA in records[i]:
                del records[i][IMG_DATA]

            if FILE in records[i]:
                del records[i][FILE]

        return records

    def custom_endpoint_processing(self, records, endpoint):
        """
        Records processing for your custom endpoint.
        :param records: list of dictionaries with _url, _file, _base64
        :param endpoint: endpoint to add to api.ximilar.com/
        :return: result from endpoint
        """
        records = self.preprocess_records(records)
        return self.post(endpoint, data={RECORDS: records})

    def get_config(self, config_type, version=None):
        """
        Get configuration json from account/v2/config endpoint
        :param config_type: name of the configuration
        :param version: number of config version (optional
        :return: json response
        """
        version = "?version=" + str(version) if version else ""
        return self.get(CONFIG_ENDPOINT + config_type + version)

    @staticmethod
    def download_image(url, destination=""):
        """
        Download image from url to the destination
        :param url: url to the image
        :param destination: where the image will be stored
        :return: None
        """
        f_name = url.split("/")[-1]
        f_dest = os.path.join(destination, f_name)
        f_dest += ".jpg" if "." not in f_name else ""

        if os.path.isfile(f_dest):
            return f_dest

        page = requests.get(url)
        with open(f_dest, "wb") as f:
            f.write(page.content)
        return f_dest

    def parallel_records_processing(self, records, method, max_workers=3, batch_size=1, output=False):
        """
        Process method which uses records in parallel way. This works for methods:

        RecogntionClient.upload_image
        RecognitionClient.classify
        FashionTaggingClient.tags
        GenericTaggingClient.tags
        DominantColorClient.dominantcolor

        :param records: list of dictionaries with _url, _file, _base64 and other metadata
                        [{'_file': '__IMG_PATH__', 'id': '__MY_IMAGE_ID__'}, ... ]
        :param method: method to call
        :param max_workers: how many threads will we spawn for work (recommended is 3)
        :param batch_size: how many images are we sending in batch
        :param output: output to stdout with progressbar / tqdm
        :return: list of results from every method
        """
        executor = concurrent.futures.ThreadPoolExecutor(max_workers=max_workers)
        futures = [
            {"future": executor.submit(method, records_to_proc), "size": len(records_to_proc)}
            for records_to_proc in self.batch(records, n=batch_size)
        ]

        results = []
        if output:
            with tqdm(total=len(records)) as pbar:
                for future in futures:
                    result = future["future"].result()
                    results.append(result)
                    pbar.update(future["size"])
        else:
            results = [future["future"].result() for future in futures]
        return results

    def batch(self, iterable, n=1):
        """
        Divides list to batches of size n.
        """
        l = len(iterable)
        for ndx in range(0, l, n):
            yield iterable[ndx : min(ndx + n, l)]

    @staticmethod
    def _convert_image_to_bgr(image, image_space="RGB"):
        """
        Converts given image from given record to BGR
        :param image: image data
        :param image_space: string saying the image space
        :return: image data in BGR
        """
        if image_space == "BGR":
            return image
        if image_space == "RGB":
            return cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
        elif image_space == "HSV":
            return cv2.cvtColor(image, cv2.COLOR_HSV2BGR)
        elif image_space == "LUV":
            return cv2.cvtColor(image, cv2.COLOR_LUV2BGR)
        raise ValueError("unknown image space: " + image_space)



from .client import RestClient
from .recognition import RecognitionClient
from .tagging import GenericTaggingClient, FashionTaggingClient
from .colors import DominantColorProductClient, DominantColorGenericClient
from .smart import SmartSearchClient
from .detection import DetectionClient
from .search import SimilarityPhotosClient, SimilarityProductsClient



from ximilar.client import RestClient
from ximilar.client.constants import *

FASHION_TAGGING_ENDPOINT = "tagging/fashion/v2/tags"
GENERIC_TAGGING_ENDPOINT = "tagging/generic/v2/tags"


class TaggingClient(RestClient):
    def tags(self, records, endpoint):
        records = self.preprocess_records(records)
        return self.post(endpoint, data={RECORDS: records})


class FashionTaggingClient(TaggingClient):
    def __init__(self, token, endpoint=ENDPOINT, resource_name=FASHION_TAGGING):
        super(FashionTaggingClient, self).__init__(token=token, endpoint=endpoint, resource_name=resource_name)

    def tags(self, records):
        return super().tags(records, FASHION_TAGGING_ENDPOINT)


class GenericTaggingClient(TaggingClient):
    def __init__(self, token, endpoint=ENDPOINT, resource_name=GENERIC_TAGGING):
        super(GenericTaggingClient, self).__init__(token=token, endpoint=endpoint, resource_name=resource_name)

    def tags(self, records):
        return super().tags(records, GENERIC_TAGGING_ENDPOINT)



import cv2
import os
from argparse import ArgumentParser

from ximilar.client import DetectionClient
from ximilar.client.constants import DEFAULT_WORKSPACE, IMG_DATA
from ximilar.client.detection import DetectionObject
from ximilar.client.recognition import Image


def resize(image_data, rect_size):
    (origH, origW) = image_data.shape[:2]

    # set the new width and height and then determine the ratio in change for both the width and height
    WHITE = [255, 255, 255]
    if origW >= origH:
        rW = rH = origW / float(rect_size)
    else:
        rW = rH = origH / float(rect_size)

    print(f"origH {origH}, origW {origW}, rW{rW}, rect_size {rect_size}")
    # resize the image
    resized_image = cv2.resize(image_data, (int(origW / rW), int(origH / rH))) if rW >= 1.0 else image_data

    # add white pixels to achieve 320 x 320
    resized_image = cv2.copyMakeBorder(
        resized_image,
        0,
        rect_size - resized_image.shape[:2][0],
        0,
        rect_size - resized_image.shape[:2][1],
        cv2.BORDER_CONSTANT,
        value=WHITE,
    )
    return resized_image


if __name__ == "__main__":
    parser = ArgumentParser(description="Get all images from a workspace and cut all their objects and create files")
    parser.add_argument("--api_prefix", type=str, help="API prefix", default="https://api.ximilar.com/")
    parser.add_argument("--auth_token", help="user authorization token to be used for API authentication")
    parser.add_argument("--workspace_id", help="ID of workspace to upload the images into", default=DEFAULT_WORKSPACE)
    parser.add_argument("--label_id", help="directory to print the images to")
    parser.add_argument("--output_dir", help="directory to print the images to")
    parser.add_argument("--resize", help="if not 0 or negative, the image is resized and filled with white", type=int)
    parser.add_argument("--print_details", help="if true, info about each image is printed out", action="store_true")

    args = parser.parse_args()

    # client = RecognitionClient(token=args.auth_token, endpoint=args.api_prefix, workspace=args.workspace_id)
    detection_client = DetectionClient(token=args.auth_token, endpoint=args.api_prefix, workspace=args.workspace_id)

    image_dir = os.path.join(args.output_dir, "images")
    os.makedirs(image_dir, exist_ok=True)

    # getting all images of label (paginated result)
    images, next_page, status = detection_client.get_training_images()
    while images:
        image: Image
        for image in images:
            objects, status = detection_client.get_objects_of_image(image.id)
            obj: DetectionObject
            i = 1
            for obj in objects:
                image_record = image.extract_object_data(obj.get_bbox(), image_dir)
                object_file = os.path.join(
                    args.output_dir, str(image.id) + "_" + obj.detection_label["name"] + "_" + str(i) + ".jpg"
                )
                if args.resize > 0:
                    image_record[IMG_DATA] = resize(image_record[IMG_DATA], args.resize)
                detection_client.cv2_imwrite(image_record, object_file)
                if args.print_details:
                    print("creating file " + str(object_file))
                i += 1

        if not next_page:
            break
        images, next_page, status = detection_client.get_training_images(next_page)



import os
import sys
from argparse import ArgumentParser

from ximilar.client import RecognitionClient
from ximilar.client.constants import FILE, DEFAULT_WORKSPACE, NORESIZE


if __name__ == "__main__":
    parser = ArgumentParser(description="Image uploader into Ximilar App")
    parser.add_argument("--api_prefix", type=str, help="API prefix", default="https://api.ximilar.com/")
    parser.add_argument("--auth_token", help="user authorization token to be used for API authentication")
    parser.add_argument("--workspace_id", help="ID of workspace to upload the images into", default=DEFAULT_WORKSPACE)
    parser.add_argument("--input_dir", help="directory with the images (recursively)")
    parser.add_argument("--no_resize", help="flag whether to preserve image size", action="store_true")
    parser.add_argument("--print_details", help="if true, info about each image is printed out", action="store_true")
    parser.add_argument(
        "--extensions",
        help="list of file extensions to consider (ignore case)",
        type=list,
        default=[".jpg", ".jpeg", ".png", ".tiff", ".tif", ".bmp", ".gif"],
    )
    args = parser.parse_args()

    client = RecognitionClient(token=args.auth_token, endpoint=args.api_prefix, workspace=args.workspace_id)

    if not os.path.isdir(args.input_dir):
        print("directory does not exist: " + args.input_dir, file=sys.stderr)
        exit(1)

    for root, subdirs, files in os.walk(args.input_dir):
        files = filter(lambda x: os.path.splitext(x)[1].lower() in args.extensions, files)
        records = [{FILE: os.path.join(root, filename), NORESIZE: args.no_resize} for filename in files]
        results = client.parallel_records_processing(records, client.upload_images, output=True)

        for (images, status) in results:
            if status["status"] != "OK":
                print("error status: " + str(status))
            if args.print_details:
                for image in images:
                    print(image.img_path + " (" + str(image.img_width) + "px x " + str(image.img_height) + "px)")



from ximilar.client import RestClient
from ximilar.client.constants import *

DOMINANT_COLOR_GENERIC_ENDPOINT = "generic_photos/v2/dominantcolor"
DOMINANT_COLOR_PRODUCT_ENDPOINT = "product_photos/v2/dominantcolor"


class DominantColorClient(RestClient):
    def dominantcolor(self, records, endpoint):
        records = self.preprocess_records(records)
        return self.post(endpoint, data={RECORDS: records})


class DominantColorGenericClient(DominantColorClient):
    def __init__(
        self, token, collection=None, collection_id=None, endpoint=ENDPOINT, resource_name=DOMINANT_COLORS_PRODUCT
    ):
        super(DominantColorGenericClient, self).__init__(
            token=token,
            collection=collection,
            collection_id=collection_id,
            endpoint=endpoint,
            resource_name=resource_name,
        )

    def dominantcolor(self, records):
        return super().dominantcolor(records, DOMINANT_COLOR_GENERIC_ENDPOINT)


class DominantColorProductClient(DominantColorClient):
    def __init__(
        self, token, collection=None, collection_id=None, endpoint=ENDPOINT, resource_name=DOMINANT_COLORS_GENERIC
    ):
        super(DominantColorProductClient, self).__init__(
            token=token,
            collection=collection,
            collection_id=collection_id,
            endpoint=endpoint,
            resource_name=resource_name,
        )

    def dominantcolor(self, records):
        return super().dominantcolor(records, DOMINANT_COLOR_PRODUCT_ENDPOINT)



from ximilar.client.constants import (
    _ID,
    K_COUNT,
    FIELDS_TO_RETURN,
    ENDPOINT,
    SMART_PRODUCT_SEARCH,
    RECORDS,
    COLLECTION,
    FILTER,
)
from ximilar.client.search import SimilarityPhotosClient

SMART_SEARCH = "smart/v2/"

SEARCH_OBJ_ENDPOINT = "product/search_by_object"
SEARCH_PRODUCT = "product/search"
DETECT_PRODUCT = "product/detect"
INSERT_PRODUCT = "product/insert"
ALL_IDS = "allIDs"
PING = "ping"


class SmartSearchClient(SimilarityPhotosClient):
    def __init__(
        self,
        token,
        collection=None,
        collection_id=None,
        endpoint=ENDPOINT + SMART_SEARCH,
        resource_name=SMART_PRODUCT_SEARCH,
    ):
        super(SmartSearchClient, self).__init__(
            token=token,
            collection=collection,
            collection_id=collection_id,
            endpoint=endpoint,
            resource_name=resource_name,
        )

    def search(self, records, filter=None, count=5, fields_to_return=[_ID]):
        """
        Detects Objects and Tags and find for the largest object most visually similar items in your collection.
        :param records: array with one record (dictionary) with '_url' or "_base64' data
        :param count: how many similar items to return
        :param fields_to_return: fields to return in every record
        :param filter: how to filter picked items (mongodb syntax)
        :return:
        """
        data = {RECORDS: self.preprocess_records(records), K_COUNT: count, FIELDS_TO_RETURN: fields_to_return}
        if filter:
            data[FILTER] = filter

        return self.post(SEARCH_PRODUCT, data=data)

    def detect(self, records):
        """
        Detects Objects and Tags without searching items.
        :param records: list of dictionaries with _url|_file|_base64
        :return: json response
        """
        records = self.preprocess_records(records)
        return self.post(DETECT_PRODUCT, data={RECORDS: records})

    def search_by_object(self, records, filter=None, count=5, fields_to_return=[_ID]):
        """
        Detects Objects and Tags without searching items.
        :param records: list of dictionaries with _url|_file|_base64
        :return: json response
        """
        data = {RECORDS: self.preprocess_records(records), K_COUNT: count, FIELDS_TO_RETURN: fields_to_return}
        if filter:
            data[FILTER] = filter
        return self.post(SEARCH_OBJ_ENDPOINT, data={RECORDS: records, COLLECTION: self.headers[COLLECTION]})

    def insert(self, records):
        """
        Insert records into collection with all meta information.
        :param records: dictionary with your "_id" and with one of "_url", "_file" or "_base64" to extract descriptor.
        :return: json response
        """
        data = {RECORDS: records}
        return self.post(INSERT_PRODUCT, data=data)

    def random(self, filter=None, count=10, fields_to_return=[]):
        raise NotImplementedError("random operation does not work for Smart Search")

    def get_all_ids(self):
        """
        Returns an array with records with "_id" fields.
        :return: an array with records with "_id" fields.
        """
        return self.post(ALL_IDS, data={"something": "value"})



from setuptools import setup, find_packages
import os

with open("requirements.txt") as f:
    install_requirements = f.read().splitlines()

if os.environ.get("CI_COMMIT_TAG"):
    version = os.environ["CI_COMMIT_TAG"]
else:
    version = "2.0.dev1"


setup(
    name="ximilar-client",
    version=version,
    description="The Ximilar App and Vize.ai Client.",
    url="https://gitlab.com/ximilar-public/ximilar-vize-api",
    author="Michal Lukac, David Novak and Ximilar.com Team",
    author_email="tech@ximilar.com",
    license="Apache 2.0",
    packages=find_packages(),
    keywords="machine learning, multimedia, json, rest, data",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.4",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    install_requires=install_requirements,
    include_package_data=True,
    zip_safe=False,
    namespace_packages=["ximilar"],
)
